@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -39,14 +39,12 @@
 #include <gccv/text.h>
 #include <gccv/text-tag.h>
 #include <gcu/element.h>
-#include <openbabel/mol.h>
 #include <glib/gi18n-lib.h>
 #include <cstdlib>
 #include <cstring>
 #include <sstream>
 
 using namespace gcu;
-using namespace OpenBabel;
 using namespace std;
 
 #define ATOM_EPSILON 0.1
@@ -126,38 +124,6 @@ Atom::Atom (int Z, double x, double y, double z):
 	SetZ(Z);
 }
 
-Atom::Atom (OBAtom* atom):
-	gcu::Atom (),
-	ItemClient (),
-	m_ShowSymbol (false),
-	m_HPosStyle (AUTO_HPOS)
-{
-	m_x = atom->GetX ();
-	m_y = - atom->GetY ();
-	m_z = atom->GetZ ();
-	m_nlp = m_nlu = 0;
-	gchar* Id = g_strdup_printf ("a%d", atom->GetIdx());
-	SetId (Id);
-	g_free (Id);
-	m_HPos = GetBestSide ();
-	m_ascent = 0;
-	m_CHeight = 0.;
-	m_Changed = 0;
-	m_AvailPosCached = false;
-	m_OccupiedPos = 0;
-	m_ChargePos = 0xff;
-	m_ChargeAngle = 0.;
-	m_ChargeDist = 0.;
-	m_ChargeAutoPos = true;
-	m_Layout = m_ChargeLayout = m_HLayout = NULL;
-	m_DrawCircle = false;
-	m_Charge = atom->GetFormalCharge ();
-	m_SWidth = 0.;
-	m_ChargeItem = NULL;
-	m_ShowCharge = true;
-	SetZ (atom->GetAtomicNum ());
-}
-
 void Atom::SetZ (int Z)
 {
 	gcu::Atom::SetZ (Z);
@@ -1080,6 +1046,25 @@ void Atom::SetCharge (int charge)
 	Update ();
 }
 
+void Atom::Move (double x, double y, double z)
+{
+	gcu::Atom::Move (x, y, z);
+	map<string, Object*>::iterator i;
+	Object* electron = GetFirstChild (i);
+	while (electron) {
+		electron->Move (x, y, z);
+		electron = GetNextChild (i);
+	}
+	if (GetCharge ()) {
+		if (m_ChargeAutoPos) {
+			if (m_ChargePos > 0)
+				NotifyPositionOccupation (m_ChargePos, false);
+			m_ChargePos = 0xff;
+			Update ();
+		}
+	}
+}
+
 void Atom::Transform2D (Matrix2D& m, double x, double y)
 {
 	gcu::Atom::Transform2D (m, x, y);
@@ -1299,11 +1284,11 @@ void Atom::BuildSymbolGeometry (double width, double height, double ascent)
 
 void Atom::AddItem ()
 {
-	if (m_Item)
+	if (m_Item || GetZ() <= 0)
 		return;
 	if (m_Changed) {
 		Update ();
-		m_Changed = true;
+		m_Changed = false;
 	}
 	Document *doc = static_cast <Document*> (GetDocument ());
 	View *view = doc->GetView ();
