@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.h 
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -107,16 +107,6 @@ Empties a document and reinitialize it.
 */
 	View* GetView () {return m_pView;}
 /*!
-@param Mol an OpenBabel molecule to import.
-
-Imports a molecule using OpenBabel.
-*/
-	bool ImportOB (OpenBabel::OBMol& Mol);
-/*!
-Exports the current file using OpenBabel. Only molecules will be exported.
-*/
-	void ExportOB () const;
-/*!
 Saves the current file.
 */
 	void Save () const;
@@ -138,7 +128,7 @@ The document title.
 */
 	void SetTitle (const gchar* title);
 /*!
-@param the new window title.
+@param label the new window title.
 
 Sets the label to use as window title.
 */
@@ -157,14 +147,15 @@ Sets the new file name and its associated mime type.
 /*!
 @return the current file name, actually the URI.
 */
-	const gchar* GetFileName () {return m_filename;}
+	const gchar* GetFileName () const {return m_filename;}
 /*!
 @param print a GtkPrintOperation.
 @param	context a GtkPrintContext.
+@param page the page to print.
 
 Prints the document.
 */
-	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context, int page) const;
 /*!
 @param pObject a new object.
 
@@ -284,8 +275,6 @@ Aborts and deletyes the current operation.
 */
 	void AbortOperation ();
 /*!
-@param operation the operaton to remove.
-
 Removes an operation from the udo stack and deletes it.
 */
 	void PopOperation ();
@@ -314,9 +303,10 @@ menus according to the document state.
 /*!
 @param node the XML node representing objects to add to the document.
 
-Adds previously serialized objects  to the document.
+Adds previously serialized objects from the clipboard to the document. Links
+can be set only to or from pasted objects.
 */
-	void AddData (xmlNodePtr node);
+	void PasteData (xmlNodePtr node);
 /*!
 @return true if the undo stack is not empty.
 */
@@ -378,7 +368,7 @@ existing bonds or size of existing text.
 */
 	void SetTheme (Theme *theme);
 /*!
-param Signal a SignalId
+@param Signal a SignalId
 @param Child the child which emitted the signal or NULL
 
 Called when a signal has been emitted by a child. Only OnThemeChangedSignal
@@ -407,12 +397,23 @@ data so that the bond lengths fit the themed bond length.
 @param property the property id as defined in objprops.h
 @param value the property value as a string
 
-Used when loading to set properties. Supported oroperties for documents are:
+Used when loading to set properties. Supported properties for documents are:
 GCU_PROP_DOC_FILENAME, GCU_PROP_DOC_MIMETYPE, GCU_PROP_DOC_TITLE,
 GCU_PROP_DOC_COMMENT, GCU_PROP_DOC_CREATOR, GCU_PROP_DOC_CREATION_TIME,
 GCU_PROP_DOC_MODIFICATION_TIME, and GCU_PROP_THEME_BOND_LENGTH.
 */
 	bool SetProperty (unsigned property, char const *value);
+
+/*!
+@param property the property id as defined in objprops.h
+
+Used when saving to get properties from document.. Supported properties for documents are:
+GCU_PROP_DOC_FILENAME, GCU_PROP_DOC_MIMETYPE, GCU_PROP_DOC_TITLE,
+GCU_PROP_DOC_COMMENT, GCU_PROP_DOC_CREATOR, GCU_PROP_DOC_CREATION_TIME,
+GCU_PROP_DOC_MODIFICATION_TIME, and GCU_PROP_THEME_BOND_LENGTH.
+*/
+	std::string GetProperty (unsigned property) const;
+
 /*!
 @param loading whether the document is loading data or not.
 
@@ -448,9 +449,9 @@ of the residue.
 */
 	gcu::Residue *CreateResidue (char const *name, char const *symbol, gcu::Molecule *molecule);
 
+	std::set <std::string> &GetNewObjects () {return m_NewObjects;}
+
 private:
-	void BuildBondList (std::list<Bond*>& BondList, gcu::Object const *obj) const;
-	void BuildAtomTable (std::map<std::string, unsigned>& AtomTable, gcu::Object const *obj, unsigned& index) const;
 	void RemoveAtom (Atom* pAtom);
 	void RemoveBond (Bond* pBond);
 	void RemoveFragment (Fragment* pFragment);
@@ -474,6 +475,7 @@ private:
 	unsigned m_LastStackSize; // undo list size when last saved
 	std::set<Residue const *> m_SavedResidues;
 	std::map<std::string, gcu::SymbolResidue> m_Residues;
+	std::set <std::string> m_NewObjects;
 
 /* Theme is not really a read only property, but we provide a special Set
 method */
@@ -608,6 +610,15 @@ Sets the rights for the document clipbard access.
 @return whether the document may use the clipboard or not as a reference.
 */
 GCU_PROP (bool, AllowClipboard)
+/*!\fn GetSoftwareVersion()
+Retrieves the GChemPaint API number corresponding to the program which saved
+the file last time. Might be 0 if the file has never been saved, or if the file
+was not saved using GChemPaint. The version is an unsigned decimal number of the
+type MMMmmmµµµ, where MMM is the major version, mmm the minor version and µµµ
+the micro version. This is used to detect files with old syntax on loading.
+@return the GChemPaint API version which last saved the file.
+*/
+GCU_RO_PROP (unsigned, SoftwareVersion)
 };
 
 }	//	namespace gcp
