@@ -4,7 +4,7 @@
  * GChemPaint library
  * molecule.cc 
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,24 +32,212 @@
 #include "tool.h"
 #include "view.h"
 #include <gcu/chain.h>
+#include <gsf/gsf-input-memory.h>
+#include <gsf/gsf-input-stdio.h>
+#include <gsf/gsf-output-memory.h>
 #include <glib/gi18n-lib.h>
 #include <unistd.h>
-#include <openbabel/obconversion.h>
-#include <clocale>
 #include <cmath>
 #include <cstring>
 #include <map>
 #include <set>
+#include <sstream>
 
 using namespace gcu;
-using namespace OpenBabel;
 using namespace std;
 
 namespace gcp {
 
-static void do_export_to_ghemical (Molecule* pMol)
+typedef struct {
+	std::string name;
+	std::string uri;
+	std::string classname;
+} BaseAccess;
+
+class MoleculePrivate
 {
-	pMol->ExportToGhemical ();
+public:
+	static void ShowInChIKey (Molecule *mol);
+	static void ShowInChI (Molecule *mol);
+	static void ShowSMILES (Molecule *mol);
+	static void ExportToGhemical (Molecule *mol);
+	static void ExportTo3D (Molecule *mol);
+	static void ExportToAvogadro (Molecule *mol);
+	static char *Build3D (Molecule *mol);
+	static void LoadDatabases (char const *filename);
+	static void ShowDatabase (GObject *action);
+
+	static std::vector < BaseAccess > Databases;
+};
+
+std::vector < BaseAccess > MoleculePrivate::Databases;
+
+void MoleculePrivate::ShowInChIKey (Molecule *mol)
+{
+
+	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChIKey (), StringDlg::INCHIKEY);
+}
+
+void MoleculePrivate::ShowInChI (Molecule *mol)
+{
+
+	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetInChI (), StringDlg::INCHI);
+}
+
+void MoleculePrivate::ShowSMILES (Molecule *mol)
+{
+
+	new StringDlg (reinterpret_cast<Document *>(mol->GetDocument ()), mol->GetSMILES (), StringDlg::SMILES);
+}
+
+char *MoleculePrivate::Build3D (Molecule *mol)
+{
+	std::string const &InChI = mol->GetInChI ();
+	GsfInput *in = gsf_input_memory_new (reinterpret_cast <guint8 const *> (InChI.c_str ()), InChI.length (), false);
+	char *cml = mol->GetDocument ()->GetApp ()->ConvertToCML (in, "inchi", "-hc --gen3D");
+	g_object_unref (in);
+	return cml;
+}
+
+void MoleculePrivate::ExportToGhemical (Molecule *mol)
+{
+	char *cml = Build3D (mol);
+	if (!cml) // how does this happen?
+		return;
+	char *tmpname = g_strdup ("/tmp/gprXXXXXX.gpr");
+	int f = g_mkstemp (tmpname);
+	close (f);
+	std::string uri = "file://";
+	uri += tmpname;
+	mol->GetDocument ()->GetApp ()->ConvertFromCML (cml, uri, "gpr");
+	g_free (cml);
+	char *command_line = g_strconcat ("ghemical -f ", tmpname, NULL);	
+	g_free (tmpname);
+	g_spawn_command_line_async (command_line, NULL);
+	g_free (command_line);
+}
+
+void MoleculePrivate::ExportTo3D (Molecule *mol)
+{
+	char *cml = Build3D (mol);
+	if (!cml) // how does this happen?
+		return;
+	char *tmpname = g_strdup ("/tmp/cmlXXXXXX.cml");
+	int f = g_mkstemp (tmpname);
+	write (f, cml, strlen (cml));
+	close (f);
+	g_free (cml);
+	char *command_line = g_strconcat ("gchem3d-"GCU_API_VER" ", tmpname, NULL);	
+	g_free (tmpname);
+	g_spawn_command_line_async (command_line, NULL);
+	g_free (command_line);
+}
+
+void MoleculePrivate::ExportToAvogadro (Molecule *mol)
+{
+	char *cml = Build3D (mol);
+	if (!cml) // how does this happen?
+		return;
+	char *tmpname = g_strdup ("/tmp/cmlXXXXXX.cml");
+	int f = g_mkstemp (tmpname);
+	write (f, cml, strlen (cml));
+	close (f);
+	g_free (cml);
+	char *command_line = g_strconcat ("avogadro ", tmpname, NULL);	
+	g_free (tmpname);
+	g_spawn_command_line_async (command_line, NULL);
+	g_free (command_line);
+}
+
+static void
+database_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+{
+	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
+	if (state->classname == "molecule" && state->name.length () && state->uri.length ())
+		MoleculePrivate::Databases.push_back (*state);
+	state->name.clear ();
+	state->uri.clear ();
+	state->classname.clear ();
+};
+
+static void
+database_name_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+{
+	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
+	state->name = _(xin->content->str); // this one might be translated
+};
+
+static void
+database_uri_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+{
+	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
+	state->uri = xin->content->str;
+};
+
+static void
+database_class_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
+{
+	BaseAccess *state = static_cast < BaseAccess * > (xin->user_state);
+	state->classname = xin->content->str;
+};
+
+static GsfXMLInNode const databases_dtd[] = {
+GSF_XML_IN_NODE (DATABASES, DATABASES, -1, "databases", GSF_XML_NO_CONTENT, NULL, NULL),
+	GSF_XML_IN_NODE (DATABASES, DATABASE, -1, "database", GSF_XML_NO_CONTENT, NULL, database_end),
+		GSF_XML_IN_NODE (DATABASE, NAME, -1, "name", GSF_XML_CONTENT, NULL, database_name_end),
+		GSF_XML_IN_NODE (DATABASE, URI, -1, "uri", GSF_XML_CONTENT, NULL, database_uri_end),
+		GSF_XML_IN_NODE (DATABASE, CLASS, -1, "class", GSF_XML_CONTENT, NULL, database_class_end),
+	GSF_XML_IN_NODE_END
+};
+
+void MoleculePrivate::LoadDatabases (char const *filename)
+{
+	GError *err = NULL;
+	GsfInput *input = gsf_input_stdio_new (filename, &err);
+	if (err) {
+		g_error_free (err);
+		return;
+	}
+	GsfXMLInDoc *xml = gsf_xml_in_doc_new (databases_dtd, NULL);
+	BaseAccess state;
+	gsf_xml_in_doc_parse (xml, input, &state);
+
+	gsf_xml_in_doc_free (xml);
+	g_object_unref (input);
+}
+
+typedef struct {
+	Molecule *mol;
+	BaseAccess *access;
+} AccessState;
+
+void MoleculePrivate::ShowDatabase (GObject *action)
+{
+	AccessState *state = static_cast < AccessState * > (g_object_get_data (action, "state"));
+	if (state) {
+		std::string uri = state->access->uri;
+		size_t pos = uri.find ('%');
+		std::string key;
+		switch (uri[pos + 1]) {
+		case 'I':
+			key = state->mol->GetInChI ();
+			break;
+		case 'K':
+			key = state->mol->GetInChIKey ();
+			break;
+		case 'S':
+			key = state->mol->GetSMILES ();
+			break;
+		default:
+			return;
+		}
+		if (key.length () == 0)
+			return;
+		char *escaped = g_uri_escape_string (key.c_str (), NULL, false);
+		uri.replace (pos, 2, escaped);
+		g_free (escaped);
+		static_cast < Document * > (state->mol->GetDocument ())->GetApplication ()->ShowURI (uri);
+	}
 }
 
 static void do_select_alignment (GObject *action, Molecule* pMol)
@@ -58,42 +246,20 @@ static void do_select_alignment (GObject *action, Molecule* pMol)
 	pMol->SelectAlignmentItem (object);
 }
 
-static void do_build_inchi (Molecule* pMol)
-{
-	pMol->ShowInChI ();
-}
-
-static void do_build_smiles (Molecule* pMol)
-{
-	pMol->BuildSMILES ();
-}
-
-static void do_show_webbook (Molecule* pMol)
-{
-	pMol->ShowWebBase ("http://webbook.nist.gov/cgi/cbook.cgi?Name=", "&Units=SI");
-}
-
-static void do_show_pubchem (Molecule* pMol)
-{
-	pMol->ShowWebBase ("http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&DB=pccompound&term=\"", "\"");
-}
-
 static void do_open_in_calc (Molecule* pMol)
 {
 	pMol->OpenCalc ();
 }
 
-Molecule::Molecule (TypeId Type): gcu::Molecule (Type)
+Molecule::Molecule (TypeId Type): gcu::Molecule (Type, gcu::ContentType2D)
 {
 	m_Alignment = NULL;
-	m_Changed = true;
 	m_IsResidue = false;
 }
 
-Molecule::Molecule (Atom* pAtom): gcu::Molecule (pAtom)
+Molecule::Molecule (Atom* pAtom): gcu::Molecule (pAtom, gcu::ContentType2D)
 {
 	m_Alignment = NULL;
-	m_Changed = true;
 	m_IsResidue = false;
 }
 
@@ -437,7 +603,6 @@ bool Molecule::Load (xmlNodePtr node)
 		if (!m_Alignment)
 			return false;
 	}
-	m_Changed = true;
 	pDoc->ObjectLoaded (this);
 	return true;
 }
@@ -504,32 +669,77 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 	g_object_unref (action);
 	bool result = false;
 	if (!m_Fragments.size ()) {
-		if (((Document*) GetDocument ())->GetApplication ()->HaveGhemical ()) {
-			action = gtk_action_new ("ghemical", _("Export molecule to Ghemical"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_export_to_ghemical), this);
+		Application *app = static_cast <Document *> (GetDocument ())->GetApplication ();
+		if (app->Have3DSupport ()) {
+			action = gtk_action_new ("open3d", _("Open 3D model in"), NULL, NULL);
 			gtk_action_group_add_action (group, action);
 			g_object_unref (action);
-			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='ghemical'/></menu></popup></ui>", -1, NULL);
+			if (app->GetHaveGhemical ()) {
+				action = gtk_action_new ("ghemical", _("Ghemical"), NULL, NULL);
+				g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ExportToGhemical), this);
+				gtk_action_group_add_action (group, action);
+				g_object_unref (action);
+				gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menu action='open3d'><menuitem action='ghemical'/></menu></menu></popup></ui>", -1, NULL);
+			}
+			if (app->GetHaveGChem3D ()) {
+				action = gtk_action_new ("gchem3d", _("GChem3D"), NULL, NULL);
+				g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ExportTo3D), this);
+				gtk_action_group_add_action (group, action);
+				g_object_unref (action);
+				gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menu action='open3d'><menuitem action='gchem3d'/></menu></menu></popup></ui>", -1, NULL);
+			}
+			if (app->GetHaveAvogadro ()) {
+				action = gtk_action_new ("avogadro", _("Avogadro"), NULL, NULL);
+				g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ExportToAvogadro), this);
+				gtk_action_group_add_action (group, action);
+				g_object_unref (action);
+				gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menu action='open3d'><menuitem action='avogadro'/></menu></menu></popup></ui>", -1, NULL);
+			}
 		}
-		if (((Document*) GetDocument ())->GetApplication ()->HaveInChI ()) {
+		// add databases submenu		
+		if (MoleculePrivate::Databases.empty ()) {
+			// load them using gsf xml support
+			// first $DATADIR/gchemutils/API_VER/databases.xml
+			MoleculePrivate::LoadDatabases (DATADIR"/gchemutils/"GCU_API_VER"/databases.xml");
+			// and now $HOME/.gchemutils/datatases.xml
+			std::string home = getenv ("HOME");
+			home += "/.gchemutils/databases.xml";
+			MoleculePrivate::LoadDatabases (home.c_str ());
+		}
+		if (!MoleculePrivate::Databases.empty ()) {
+			action = gtk_action_new ("database", _("Find in databases"), NULL, NULL);
+			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+			std::vector < BaseAccess >::iterator it, end = MoleculePrivate::Databases.end ();
+			for (it = MoleculePrivate::Databases.begin (); it != end; it++) {
+				AccessState *state = g_new0 (AccessState, 1);
+				state->access = &(*it);
+				state->mol = this;
+				action = gtk_action_new ((*it).name.c_str (), (*it).name.c_str (), NULL, NULL);
+				g_object_set_data_full (G_OBJECT (action), "state", state, g_free);
+				g_signal_connect (action, "activate", G_CALLBACK (MoleculePrivate::ShowDatabase), NULL);
+				gtk_action_group_add_action (group, action);
+				g_object_unref (action);
+				std::string node = "<ui><popup><menu action='Molecule'><menu action='database'><menuitem action='";
+				node += (*it).name;
+				node += "'/></menu></menu></popup></ui>";
+				gtk_ui_manager_add_ui_from_string (UIManager, node.c_str (), -1, NULL);
+			}
+		}
+		//		if (((Document*) GetDocument ())->GetApplication ()->HaveInChI ()) {
 			action = gtk_action_new ("inchi", _("Generate InChI"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_inchi), this);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChI), this);
 			gtk_action_group_add_action (group, action);
 			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='inchi'/></menu></popup></ui>", -1, NULL);
-			action = gtk_action_new ("webbook", _("NIST WebBook page for this molecule"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_webbook), this);
+			action = gtk_action_new ("inchikey", _("Generate InChIKey"), NULL, NULL);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowInChIKey), this);
 			gtk_action_group_add_action (group, action);
 			g_object_unref (action);
-			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='webbook'/></menu></popup></ui>", -1, NULL);
-			action = gtk_action_new ("pubchem", _("PubChem page for this molecule"), NULL, NULL);
-			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_pubchem), this);
-			gtk_action_group_add_action (group, action);
-			g_object_unref (action);
-			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='pubchem'/></menu></popup></ui>", -1, NULL);
-		}
+			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='inchikey'/></menu></popup></ui>", -1, NULL);
+//		}
 		action = gtk_action_new ("smiles", _("Generate SMILES"), NULL, NULL);
-		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_smiles), this);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (MoleculePrivate::ShowSMILES), this);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='smiles'/></menu></popup></ui>", -1, NULL);
@@ -554,32 +764,6 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 	return result | Object::BuildContextualMenu (UIManager, object, x, y);
 }
 
-void Molecule::ExportToGhemical ()
-{
-	OBMol Mol;
-	OBConversion Conv;
-	OBFormat* pOutFormat = Conv.FindFormat ("gpr");
-	Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
-	BuildOBMol (Mol);
-	char *tmpname = g_strdup ("/tmp/2gprXXXXXX");
-	int f = g_mkstemp (tmpname);
-	gchar *old_num_locale;
-	close (f);
-	ofstream ofs;
-	ofs.open(tmpname);
-	if (!ofs) throw (int) 1;
-	old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-	setlocale(LC_NUMERIC, "C");
-	Conv.Write (&Mol, &ofs);
-	setlocale(LC_NUMERIC, old_num_locale);
-	g_free(old_num_locale);
-	ofs.close();
-	char *command_line = g_strconcat ("ghemical -f ", tmpname, NULL);	
-	g_free (tmpname);
-	g_spawn_command_line_async (command_line, NULL);
-	g_free (command_line);
-}
-
 void Molecule::SelectAlignmentItem (Object *child)
 {
 	m_Alignment = (m_Alignment != child)? child: NULL;
@@ -596,212 +780,9 @@ xmlNodePtr Molecule::Save (xmlDocPtr xml) const
 	return node;
 }
 
-void Molecule::BuildOBMol (OBMol &Mol)
-{
-	double xav = 0., yav = 0., zf;
-	unsigned n = m_Atoms.size ();
-	map<string, unsigned> AtomTable;
-	list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
-	list<gcu::Bond*> BondList;
-	double x, y, z;
-	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		(*ia)->GetCoords(&x, &y, &z);
-		xav += x;
-		yav += y;
-	}
-	xav /= n;
-	yav /= n;
-	Atom* pgAtom;
-	OBAtom obAtom;
-	unsigned index = 1;
-	map<gcu::Atom*, gcu::Bond*>::iterator i;
-	Bond *pBond;
-	Mol.BeginModify ();
-	Mol.ReserveAtoms (n);
-	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		pgAtom =  reinterpret_cast <Atom *> (*ia);
-		AtomTable [pgAtom->GetId ()] = index;
-		obAtom.SetIdx (index++);
-		obAtom.SetAtomicNum (pgAtom->GetZ());
-		obAtom.SetFormalCharge (pgAtom->GetCharge ());
-		pgAtom->GetCoords (&x, &y, &z);
-		// Scans the atom bonds and change z to try conservation of stereochemistry
-		pBond = (Bond*) pgAtom->GetFirstBond (i);
-		while (pBond) {
-			zf = (pBond->GetAtom (0) == pgAtom)? 1.: -1.;
-			switch (pBond->GetType ()) {
-			case UpBondType:
-				z += zf * 50.;
-				break;
-			case DownBondType:
-				z -= zf * 50.;
-				break;
-			default:
-				break;
-			}
-			pBond = (Bond*) pgAtom->GetNextBond (i);
-		}
-		obAtom.SetVector ((xav - x) / 100, (yav - y) / 100, z / 100);
-		Mol.AddAtom (obAtom);
-		obAtom.Clear ();
-	}
-	list<gcu::Bond*>::iterator j, endb = m_Bonds.end ();
-	int start, end, order;
-	for (j = m_Bonds.begin (); j != endb; j++)
-	{
-		order = (*j)->GetOrder ();
-		start = AtomTable[(*j)->GetAtom (0)->GetId ()];
-		end = AtomTable[(*j)->GetAtom (1)->GetId ()];
-		Mol.AddBond(start, end, order, 0);
-	}
-	Mol.EndModify ();
-}
-
-void Molecule::BuildOBMol2D (OBMol &Mol)
-{
-	double xav = 0., yav = 0.;
-	unsigned n = m_Atoms.size ();
-	map<string, unsigned> AtomTable;
-	list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
-	list<gcu::Bond*> BondList;
-	double x, y, z;
-	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		(*ia)->GetCoords (&x, &y, &z);
-		xav += x;
-		yav += y;
-	}
-	xav /= n;
-	yav /= n;
-	Atom* pgAtom;
-	OBAtom obAtom;
-	unsigned index = 1;
-	map<gcu::Atom*, gcu::Bond*>::iterator i;
-	Mol.BeginModify ();
-	Mol.ReserveAtoms (n);
-	Mol.SetDimension (2);
-	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		pgAtom =  reinterpret_cast <Atom *> (*ia);
-		AtomTable [pgAtom->GetId ()] = index;
-		obAtom.SetIdx (index++);
-		obAtom.SetAtomicNum (pgAtom->GetZ());
-		obAtom.SetFormalCharge (pgAtom->GetCharge ());
-		pgAtom->GetCoords (&x, &y, &z);
-		// Scans the atom bonds and change z to try conservation of stereochemistry
-		obAtom.SetVector ((x - xav) / 100, (yav - y) / 100, 0.);
-		Mol.AddAtom (obAtom);
-		obAtom.Clear ();
-	}
-	list<gcu::Bond*>::iterator j, endb = m_Bonds.end ();
-	int start, end, order, flag;
-	for (j = m_Bonds.begin (); j != endb; j++) {
-		order = (*j)->GetOrder ();
-		start = AtomTable[(*j)->GetAtom (0)->GetId ()];
-		end = AtomTable[(*j)->GetAtom (1)->GetId ()];
-		switch (reinterpret_cast <Bond *> (*j)->GetType ()) {
-		case UpBondType:
-			flag = OB_WEDGE_BOND;
-			break;
-		case DownBondType:
-			flag = OB_HASH_BOND;
-			break;
-		default:
-			flag = 0;
-		}
-		Mol.AddBond (start, end, order, flag);
-	}
-	Mol.EndModify ();
-}
-
-void Molecule::BuildInChI ()
-{
-	OBMol Mol;
-	OBConversion Conv;
-	BuildOBMol2D (Mol);
-	OBFormat *pInChIFormat = Conv.FindFormat ("inchi"), *pMolFormat = Conv.FindFormat ("mol");
-	if (pInChIFormat) {
-		Conv.SetInAndOutFormats (pMolFormat, pInChIFormat);
-		Conv.SetOptions ("xt", OpenBabel::OBConversion::OUTOPTIONS);
-		ostringstream ofs;
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
-		Conv.Write (&Mol, &ofs);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
-		// remove "INCHI=" and the new line char at the end
-		m_InChI = ofs.str ().substr (0, ofs.str ().length () - 2);
-	} else {
-		Conv.SetInAndOutFormats (pMolFormat, pMolFormat);
-		char *tmpname = g_strdup ("/tmp/inchiXXXXXX");
-		int f = g_mkstemp (tmpname);
-		close (f);
-		ofstream ofs;
-		ofs.open (tmpname);
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
-		Conv.Write (&Mol, &ofs);
-		setlocale (LC_NUMERIC, old_num_locale);
-		ofs.close ();
-		// calling main_inchi -STDIO -AuxNone -NoLabels
-		char *command = g_strdup_printf ("main_inchi %s -STDIO -AuxNone -NoLabels", tmpname);
-		char *output, *errors;
-		g_spawn_command_line_sync (command, &output, &errors, NULL, NULL);
-		if (output) {
-			// remove the new line char at the end
-			output[strlen (output) - 1] = 0;
-			m_InChI = output + 6;
-			g_free (output);
-		}
-		if (errors)
-			g_free (errors);
-		g_free (command);
-		g_free (old_num_locale);
-		remove (tmpname);
-		g_free (tmpname);
-	}
-	m_Changed = false;
-}
-
-void Molecule::BuildSMILES ()
-{
-	OBMol Mol;
-	OBConversion Conv;
-	OBFormat* pOutFormat = Conv.FindFormat ("smi");
-	Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
-	BuildOBMol2D (Mol);
-	ostringstream ofs;
-	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-	Conv.Write (&Mol, &ofs);
-	setlocale (LC_NUMERIC, old_num_locale);
-	//TODO: do something with the string
-	g_free (old_num_locale);
-	string str = ofs.str ().substr (0, ofs.str ().length () - 2);
-	new StringDlg (reinterpret_cast<Document *>(GetDocument ()), str, StringDlg::SMILES);
-}
-
-void Molecule::ShowInChI ()
-{
-	if (m_Changed)
-		BuildInChI ();
-	new StringDlg (reinterpret_cast<Document *>(GetDocument ()), m_InChI, StringDlg::INCHI);
-}
-
-void Molecule::ShowWebBase (char const* uri_start, char const *uri_end)
-{
-	if (m_Changed)
-		BuildInChI ();
-	if (m_InChI.length () == 0)
-		return; //should emit at least a warning
-	string::size_type t;
-	while ((t = m_InChI.find ('+')) != string::npos)
-		m_InChI.replace (t, 1, "%2b");
-	string uri = string (uri_start) + m_InChI + uri_end;
-	((Document*) GetDocument ())->GetApplication ()->ShowURI (uri);
-}
-
 bool Molecule::OnSignal (G_GNUC_UNUSED SignalId Signal, G_GNUC_UNUSED Object *Child)
 {
-	m_Changed = true;
+	ResetIndentifiers ();
 	return true;
 }
 
@@ -832,13 +813,6 @@ void Molecule::CheckCrossings (Bond *pBond)
 		}
 }
 
-char const *Molecule::GetInChI ()
-{
-	if (m_Changed)
-		BuildInChI ();
-	return m_InChI.c_str ();
-}
-
 std::string Molecule::GetRawFormula () const
 {
 	ostringstream ofs;
