@@ -4,7 +4,7 @@
  * GChemPaint library
  * molecule.h 
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,7 +28,6 @@
 #include "fragment.h"
 #include <list>
 #include <gcu/molecule.h>
-#include <openbabel/mol.h>
 
 /*!\file*/
 namespace gcp {
@@ -40,6 +39,7 @@ class Bond;
 */
 class Molecule: public gcu::Molecule
 {
+friend class MoleculePrivate;
 public:
 /*!
 The default constructor. Builds a new empty molecule.
@@ -81,13 +81,6 @@ Adds a bond to the molecule.
 */
 	void AddBond (gcu::Bond* pBond);
 /*!
-@param w the GtkWidget inside which the molecule will be displayed.
-
-Used to add a representation of the molecule in the widget. It just calls
-the method for all its children.
-*/
-	void Add (GtkWidget* w) const;
-/*!
 @param pObject an atom, a bond, or a fragment in the molecule.
 
 Removes an atom, a bond, or a fragment from a molecule.
@@ -165,11 +158,6 @@ Signals the molecule that at least one of its children changed.
 */
 	bool OnSignal (gcu::SignalId Signal, gcu::Object *Child);
 /*!
-Opens a Ghemical window with a copy of the molecule trying to preserve the
-stereochemistry as far as possible.
-*/
-	void ExportToGhemical ();
-/*!
 @param child the child used for alignment.
 
 Selects the child used for alignment, which might be an atom or group of atoms,
@@ -182,43 +170,6 @@ a call to their gcu::Object::GetYAlign() method will be used.
 */
 	std::string GetAlignmentId () {return (m_Alignment)? m_Alignment->GetId (): "";}
 /*!
-@param Mol the OpenBabel molecule used for export.
-
-Builds an OpenBabel molecule from this instance.
-The new molecule is built trying to guess appropriate z coordinates so that
-stereochemistry is preserved.
-*/
-	void BuildOBMol (OpenBabel::OBMol &Mol);
-/*!
-@param Mol the OpenBabel molecule used for export.
-
-Builds a 2D OpenBabel molecule from this instance.
-
-*/
-	void BuildOBMol2D (OpenBabel::OBMol &Mol);
-/*!
-Builds the InChI for the molecule if necessary and opens a new StringDlg instance
-displaying it.
-*/
-	void ShowInChI ();
-/*!
-Builds the InChI for the molecule.
-*/
-	void BuildInChI ();
-/*!
-Builds the SMILES representation for the molecule and opens a new StringDlg
-instance displaying it.
-*/
-	void BuildSMILES ();
-/*!
-@param uri_start the first part of the URI to open.
-@param uri_end the last part of the URI to open.
-
-Opens a web browser with an URI constructed from both arguments and the InChI
-for the molecule ntercalated between \a uri_start and \a uri_end.
-*/
-	void ShowWebBase (char const *uri_start, char const *uri_end);
-/*!
 Opens GChemCalc with the formula for the molecule. Molecules with fragments
 are not currently supported.
 */
@@ -231,11 +182,6 @@ that they are crossing.
 */
 	void CheckCrossings (Bond *pBond);
 /*!
-@return the InChI. The returned string should not be freed (it's a const char*).
-Molecules with fragments are not currently supported.
-*/
-	char const *GetInChI ();
-/*!
 @return the raw formula as a string. Molecules with fragments
 are not currently supported.
 */
@@ -253,8 +199,6 @@ only, whatever their real atomic composition.
 private:
 	std::list<Fragment*> m_Fragments;
 	gcu::Object *m_Alignment;
-	std::string m_InChI;
-	bool m_Changed;
 	bool m_IsResidue;
 };
 
