@@ -4,7 +4,7 @@
  * GChemPaint library
  * widgetdata.cc
  *
- * Copyright (C) 2002-20067 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -378,8 +378,10 @@ void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect &rect) const
 	gccv::ItemClient const *client, *child_client;
 	double x1, y1, x2,  y2;
 	client = dynamic_cast <gccv::ItemClient const *> (obj);
-	if (client && client->GetItem ()) {
-		client->GetItem ()->GetBounds (x1, y1, x2, y2);
+	gccv::Item const *item;
+	// will not work if the object item is not top level, but can this happen?
+	if (client && (item = client->GetItem ()) && item->IsTopLevel ()) {
+		item->GetBounds (x1, y1, x2, y2);
 		if (x2 > 0.) {
 			if (!go_finite (rect.x0)) {
 				rect.x0 = x1;
@@ -422,6 +424,16 @@ void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect *rect) const
 		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
 }
 
+void WidgetData::GetObjectsBounds (std::set <gcu::Object const *> const &objects, gccv::Rect *rect) const
+{
+	rect->x0 = go_nan;
+	std::set <gcu::Object const *>::iterator it, end = objects.end ();
+	for (it = objects.begin (); it != end; it++)
+		GetObjectBounds (*it, *rect);
+	if (!go_finite (rect->x0))
+		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
+}
+
 void WidgetData::SelectAll ()
 {
 	std::list <gccv::Item *>::iterator it;
