@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/application.h
  *
- * Copyright (C) 2005-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -41,6 +41,12 @@ class Document;
 class Dialog;
 struct option_data;
 class TypeDesc;
+class CmdContext;
+
+typedef struct {
+	std::string name;
+	std::string uri;
+} Database;
 
 /*!
 Window states
@@ -86,7 +92,7 @@ the help_name or name parameters will be used.
 The datadir variable is used to build the full path to the help file:
 "file://"+datadir+"/gnome/help/"+name+"/"+LANG+"/"+name".xml".
 */
-	Application (std::string name, std::string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL);
+	Application (std::string name, std::string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL, CmdContext *cc = NULL);
 /*!
 The destructor.
 */
@@ -108,7 +114,7 @@ helpfilename\#myapp-mytag.
 /*!
 @return the name of the application. Mostly useful to set the icon name of a new window.
 */
-	std::string &GetName () {return Name;}
+	std::string const &GetName () const {return Name;}
 
 /*!
 @return a GtkWindow if any. Should be overloaded by children classes.
@@ -216,21 +222,45 @@ is not supported by GdkPixbuf.
 
 This method loads a document using the appropriate gcu::Loader class
 instance.
-@return true if no error occurred.
+@return the found \a ContentType ot ContentTypeUnknown if an error occured.
 */
-	ContentType Load (std::string const &uri, const gchar *mime_type, Document* Doc);
+	ContentType Load (std::string const &uri, const gchar *mime_type, Document* Doc, const char *options = NULL);
+	             
+/*!
+@param input a GsfInput.
+@param mime_type the mime type of the document.
+@param Doc the document instance which will contain the loaded data.
 
+This method loads a document using the appropriate gcu::Loader class
+instance.
+@return the found \a ContentType ot ContentTypeUnknown if an error occured.
+*/
+	ContentType Load (GsfInput *input, const gchar *mime_type, Document* Doc, const char *options = NULL);
+	            
 /*!
 @param uri the uri to which the document should be saved.
 @param mime_type the mime type of the document.
-@param Doc the document instance which contains the data to be saved.
+@param Obj the object instance which contains the data to be saved.
 @param type the type of the data to be saved (see gcu::ContentType).
 
 This method saves the document using the appropriate gcu::Loader class
 instance.
 @return true if no error occurred.
 */
-	bool Save (std::string const &uri, const gchar *mime_type, Document const *Doc, ContentType type);
+	bool Save (std::string const &uri, const gchar *mime_type, Object const *Obj, ContentType type, const char *options = NULL);
+	            
+/*!
+@param output a GsfOutput.
+@param mime_type the mime type of the document.
+@param Obj the object instance which contains the data to be saved.
+@param type the type of the data to be saved (see gcu::ContentType).
+
+This method saves the document using the appropriate gcu::Loader class
+instance using \a output as target.
+@return true if no error occurred.
+*/
+	bool Save (GsfOutput *output, const gchar *mime_type, Object const *Obj, ContentType type, const char *options = NULL);
+
 
 /*!
 Virtual method used to create documents. Default behavior does nothing and returns NULL.
@@ -377,6 +407,13 @@ adds a callback for modifying the contextual menu of objects of type Id.
 	void AddMenuCallback (TypeId Id, BuildMenuCb cb);
 
 	TypeDesc const *GetTypeDescription (TypeId Id);
+	CmdContext *GetCmdContext ();
+
+	char* ConvertToCML (std::string const &uri, const char *mime_type, const char *options = NULL);
+	char* ConvertToCML (GsfInput *input, const char *mime_type, const char *options = NULL);
+	void ConvertFromCML (const char *cml, std::string const &uri, const char *mime_type, const char *options = NULL);
+	void ConvertFromCML (const char *cml, GsfOutput *output, const char *mime_type, const char *options = NULL);
+	std::list < Database > const &GetDatabases (char const *classname) {return m_Databases[classname];}
 
 protected:
 
@@ -387,9 +424,13 @@ the program. Derived class might overide this method to change this.
 */
 	virtual void NoMoreDocsEvent () {gtk_main_quit ();}
 
+	void RegisterBabelType (const char *mime_type, const char *type);
+
 private:
 	void AddDocument (Document *Doc) {m_Docs.insert (Doc);}
 	void RemoveDocument (Document *Doc);
+	int OpenBabelSocket ();
+	char const *MimeToBabelType (char const *mime_type);
 
 private:
 	std::string Name;
@@ -399,16 +440,21 @@ private:
 	std::string CurDir;
 	std::string IconName;
 	static GOConfNode *m_ConfDir;
-	std::list<option_data> m_Options;
+	std::list <option_data> m_Options;
 	static WindowState DefaultWindowState;
-	std::map<TypeId, TypeDesc> m_Types;
+	std::map <TypeId, TypeDesc> m_Types;
+	std::map <std::string, std::string> m_BabelTypes;
 
 protected:
 /*!
 std::map of the supported pixbuf formats. Keys are the mime type names.
 */
-	std::map<std::string, GdkPixbufFormat*> m_SupportedPixbufFormats;	
+	std::map<std::string, GdkPixbufFormat*> m_SupportedPixbufFormats;
 
+	CmdContext *m_CmdContext;
+
+private:
+	std::map < std::string, std::list <Database> >m_Databases;
 /*!\var m_Docs
 The currently opened documents.
 */
