@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * atom.cc
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,7 +28,9 @@
 #include "bond.h"
 #include "document.h"
 #include "objprops.h"
+#include "vector.h"
 #include "xml-utils.h"
+#include <glib/gi18n-lib.h>
 #include <cmath>
 #include <cstring>
 #include <sstream>
@@ -206,15 +208,16 @@ bool Atom::Load (xmlNodePtr node)
 		m_Charge = 0;
 	if (!ReadPosition (node, NULL, &m_x, &m_y, &m_z) || (!LoadNode (node))) 
 		return false;
+	GetDocument ()->ObjectLoaded (this);
 	return true;
 }
 
-bool Atom::LoadNode (xmlNodePtr node)
+bool Atom::LoadNode (G_GNUC_UNUSED xmlNodePtr node)
 {
 	return true;
 }
 
-bool Atom::SaveNode (xmlDocPtr xml, xmlNodePtr node) const
+bool Atom::SaveNode (G_GNUC_UNUSED xmlDocPtr xml, G_GNUC_UNUSED xmlNodePtr node) const
 {
 	return true;
 }
@@ -231,7 +234,8 @@ bool Atom::SetProperty (unsigned property, char const *value)
 	switch (property) {
 	case GCU_PROP_POS2D: {
 		double x, y;
-		sscanf (value, "%lg %lg", &x, &y);
+		istringstream is (value);
+		is >> x >> y;
 		Document *doc = GetDocument ();
 		if (doc) {
 			x *= doc->GetScale ();
@@ -240,6 +244,15 @@ bool Atom::SetProperty (unsigned property, char const *value)
 		SetCoords (x, y);
 		break;
 	}
+	case GCU_PROP_X:
+		m_x = g_ascii_strtod (value, NULL) * GetDocument ()->GetScale ();
+		break;
+	case GCU_PROP_Y:
+		m_y = g_ascii_strtod (value, NULL) * GetDocument ()->GetScale ();
+		break;
+	case GCU_PROP_Z:
+		m_z = g_ascii_strtod (value, NULL) * GetDocument ()->GetScale ();
+		break;
 	case GCU_PROP_ID: {
 		char *Id = (*value == 'a')? g_strdup (value): g_strdup_printf ("a%s", value);
 		SetId (Id);
@@ -270,8 +283,25 @@ string Atom::GetProperty (unsigned property) const
 			res << m_x << " " << m_y;
 		break;
 	}
+	case GCU_PROP_POS3D: {
+		Document *doc = const_cast <Atom*> (this)->GetDocument ();
+		if (doc)
+			res << m_x / doc->GetScale () << " " << m_y / doc->GetScale () << " " << m_z / doc->GetScale ();
+		else
+			res << m_x << " " << m_y << " " << m_z;
+		break;
+	}
+	case GCU_PROP_ATOM_Z:
+		res << GetZ ();
+		break;
+	case GCU_PROP_ATOM_SYMBOL:
+		res << GetSymbol ();
+		break;
+	case GCU_PROP_ATOM_CHARGE:
+		res << static_cast <int> (m_Charge);
+		break;
 	default:
-			return Object::GetProperty (property);
+		return Object::GetProperty (property);
 	}
 	return res.str ();
 }
@@ -324,4 +354,14 @@ bool Atom::Match (Atom *atom, AtomMatchState &state)
 	return false;
 }
 
+Vector Atom::GetVector () const
+{
+	return Vector (m_x, m_y, m_z);
+}
+
+std::string Atom::Name ()
+{
+	return _("Atom");
+}
+
 }	//	namespace gcu
