@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/chem3ddoc.cc
  *
- * Copyright (C) 2006-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,17 +34,18 @@
 #include "vector.h"
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
-#include <openbabel/obconversion.h>
+#include <gsf/gsf-input-memory.h>
+#include <goffice/goffice.h>
 #include <gio/gio.h>
 #include <GL/gl.h>
 #include <libintl.h>
 #include <clocale>
 #include <fstream>
+#include <iostream>
 #include <ostream>
 #include <sstream>
 #include <cstring>
 
-using namespace OpenBabel;
 using namespace std;
 
 namespace gcu
@@ -82,6 +83,7 @@ static Object* CreateMolecule ()
 {
 	return new Molecule ();
 }
+
 void Chem3dDoc::Load (char const *uri, char const *mime_type)
 {
 	GVfs *vfs = g_vfs_get_default ();
@@ -100,23 +102,29 @@ void Chem3dDoc::Load (char const *uri, char const *mime_type)
 		return;
 	}
 	if (!mime_type)
-		mime_type = g_file_info_get_content_type(info);
+		mime_type = g_file_info_get_content_type (info);
+	g_object_unref (info);
+	if (!mime_type) {
+		// should not happen
+		g_object_unref (file);
+		return;
+	}
 	// try using the loader mechanism
 	Application *app = GetApp ();
-	Object *obj = Object::CreateObject ("atom", this);
+	Object *obj = app->CreateObject ("atom", this);
 	if (obj)
 		delete obj;
 	else {
-		gcu::Loader::Init (); // can be called many times
-		Object::AddType ("atom", CreateAtom, AtomType);
-		Object::AddType ("bond", CreateBond, BondType);
-		Object::AddType ("molecule", CreateMolecule, MoleculeType);
+		gcu::Loader::Init (app); // can be called many times
+		app->AddType ("atom", CreateAtom, AtomType);
+		app->AddType ("bond", CreateBond, BondType);
+		app->AddType ("molecule", CreateMolecule, MoleculeType);
 	}
 	string filename = uri;
 	Clear ();
 	ContentType type = app->Load (filename, mime_type, this);
+	Loaded ();
 	if (type == ContentType3D) {
-		g_object_unref (file);
 		// center the scene around 0,0,0
 		std::map<std::string, Object*>::iterator it;
 		obj =  GetFirstChild (it);
@@ -142,103 +150,74 @@ void Chem3dDoc::Load (char const *uri, char const *mime_type)
 		if (title)
 			SetTitle (title);
 		m_View->Update ();
-		return;
 	} else if (type != ContentTypeUnknown) {
 		Clear ();
-		g_object_unref (file);
 		// TODO: process the error (display a message at least or open with an appropriate application)
-		return;
 	}
-	gsize size = g_file_info_get_size (info);
-	g_object_unref (info);
-	GInputStream *input = G_INPUT_STREAM (g_file_read (file, NULL, &error));
-	if (error) {
-		g_message ("GIO could not create the stream: %s", error->message);
-		g_error_free (error);
-		g_object_unref (file);
-		error = NULL;
-		return;
-	}
-	gchar *buf = new gchar[size + 1];
-	gsize n = g_input_stream_read (input, buf, size, NULL, &error);
-	if (error) {
-		g_message ("GIO could not read the file: %s", error->message);
-		g_error_free (error);
-		delete [] buf;
-		g_object_unref (input);
-		g_object_unref (file);
-		error = NULL;
-		return;
-	}
-	buf[size] = 0;
-	m_Title.clear ();
-	m_Mol = new Molecule ();
-	AddChild (m_Mol);
-	if (n == size) {
-		LoadData (buf, mime_type);
-		if (m_App) {
-			char *dirname = g_path_get_dirname (uri);
-			m_App->SetCurDir (dirname);
-			g_free (dirname);
-		}
-		if (m_Title.length () == 0) {
-			char *name = g_path_get_basename (uri);
-			SetTitle (name);
-			g_free (name);
-		}
-	}
-	delete [] buf;
-	g_object_unref (input);
 	g_object_unref (file);
 }
 
-void Chem3dDoc::LoadData (char const *data, char const *mime_type)
+void Chem3dDoc::LoadData (char const *data, char const *mime_type, size_t size)
 {
-	istringstream is (data);
-	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-	OBConversion Conv;
-	OBFormat* pInFormat = Conv.FormatFromMIME (mime_type);
-	OBMol Molecule;
-	if (pInFormat) {
-		Conv.SetInAndOutFormats (pInFormat, pInFormat);
-		Conv.Read (&Molecule,&is);
-		m_Empty = Molecule.NumAtoms () == 0;
+	bool need_free = false;
+	if (!mime_type) {
+		mime_type = go_get_mime_type_for_data (data, size);
+		need_free = true;
 	}
-	setlocale (LC_NUMERIC, old_num_locale);
-	m_Mol->Clear ();
-	// center the molecule around 0,0,0
-	std::vector < OBNodeBase * >::iterator i;
-	OBAtom* atom = Molecule.BeginAtom (i);
-	std::map <OBAtom *, Atom *> atomMap;
-	gdouble x0, y0, z0;
-	x0 = y0 = z0 = 0.0;
-	while (atom) {
-		x0 += atom->GetX ();
-		y0 += atom->GetY ();
-		z0 += atom->GetZ ();
-		atom = Molecule.NextAtom (i);
+	if (!mime_type) {
+		// should not happen
+		return;
 	}
-	x0 /= Molecule.NumAtoms ();
-	y0 /= Molecule.NumAtoms ();
-	z0 /= Molecule.NumAtoms ();
-	vector3 v(-x0, -y0, -z0);
-	atom = Molecule.BeginAtom (i);
-	while (atom) {
-		atom->SetVector ((atom->GetVector () + v) * 100);
-		atomMap[atom] = new Atom (atom->GetAtomicNum (), atom->GetX (), atom->GetY (), atom->GetZ ());
-		m_Mol->AddAtom (atomMap[atom]);
-		atom = Molecule.NextAtom (i);
+	if (size == 0)
+		size = strlen (data);
+	GsfInput *input = gsf_input_memory_new (reinterpret_cast <guint8 const *> (data), size, false);
+	// try using the loader mechanism
+	Application *app = GetApp ();
+	Object *obj = app->CreateObject ("atom", this);
+	if (obj)
+		delete obj;
+	else {
+		gcu::Loader::Init (app); // can be called many times
+		app->AddType ("atom", CreateAtom, AtomType);
+		app->AddType ("bond", CreateBond, BondType);
+		app->AddType ("molecule", CreateMolecule, MoleculeType);
 	}
-	std::vector < OBEdgeBase * >::iterator j;
-	OBBond* bond = Molecule.BeginBond (j);
-	while (bond) {
-		m_Mol->AddBond (new Bond (atomMap[bond->GetBeginAtom ()], atomMap[bond->GetEndAtom ()], bond->GetBondOrder ()));
-		bond = Molecule.NextBond (j);
+	Clear ();
+	ContentType type = app->Load (input, mime_type, this);
+	Loaded ();
+	if (type == ContentType3D) {
+		// center the scene around 0,0,0
+		std::map<std::string, Object*>::iterator it;
+		obj =  GetFirstChild (it);
+		while (obj) {
+			m_Mol = dynamic_cast <Molecule *> (obj);
+			if (m_Mol)
+				break;
+			obj = GetNextChild (it);
+		}
+		// FIXME: we show only one molecule
+		double x0, y0, z0;
+		x0 = y0 = z0 = 0.0;
+		std::list <Atom *>::const_iterator i;
+		Atom const *atom = m_Mol->GetFirstAtom (i);
+		while (atom) {
+			x0 += atom->x ();
+			y0 += atom->y ();
+			z0 += atom->z ();
+			atom = m_Mol->GetNextAtom (i);
+		}
+		m_Mol->Move (-x0 * m_Mol->GetAtomsNumber (), -y0 * m_Mol->GetAtomsNumber (), -z0 * m_Mol->GetAtomsNumber ());
+		char const *title = m_Mol->GetName ();
+		if (title)
+			SetTitle (title);
+		m_View->Update ();
+	} else if (type != ContentTypeUnknown) {
+		Clear ();
+		// TODO: process the error
 	}
-	SetTitle (Molecule.GetTitle());
-	m_View->Update ();
-	g_free (old_num_locale);
+	if (need_free)
+		g_free (const_cast <char *> (mime_type));
+	g_object_unref (input);
 }
 
 struct VrmlBond {
@@ -252,7 +231,6 @@ void Chem3dDoc::OnExportVRML (string const &filename)
 {
 	if (!m_Mol)
 		return;
-	char *old_num_locale;
 	double R, w, x, y, z, x0, y0, z0, dist;
 	int n = 0, Z;
 	const gdouble* color;
@@ -269,8 +247,6 @@ void Chem3dDoc::OnExportVRML (string const &filename)
 		g_object_unref (stream);
 		return;
 	}
-	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
 
 	file << "#VRML V2.0 utf8" << endl;
 	
@@ -289,6 +265,12 @@ void Chem3dDoc::OnExportVRML (string const &filename)
 	z0 /= m_Mol->GetAtomsNumber ();
 
 	//Create prototypes for atoms
+	GcuAtomicRadius radius;
+	radius.type = GCU_VAN_DER_WAALS;
+	radius.charge = 0;
+	radius.cn = -1;
+	radius.spin = GCU_N_A_SPIN;
+	radius.scale = NULL;
 	for (atom = m_Mol->GetFirstAtom (i); atom; atom = m_Mol->GetNextAtom (i)) {
 		Z = atom->GetZ ();
 		if (!Z)
@@ -296,7 +278,9 @@ void Chem3dDoc::OnExportVRML (string const &filename)
 		symbol = Element::Symbol (Z);
 		if (AtomsMap[symbol].l.empty()) {
 			AtomsMap[symbol].n = n;
-			R = etab.GetVdwRad (Z);
+			radius.Z = Z;
+			gcu_element_get_radius (&radius);
+			R = radius.value.value / 100; // convert from pm to Angstrom (supposing that the unit is actually pm).
 			if (m_Display3D == BALL_AND_STICK)
 				R *= 0.2;
 			color = gcu_element_get_default_color (Z);
@@ -395,8 +379,6 @@ void Chem3dDoc::OnExportVRML (string const &filename)
 	//end of the world
 	file << "\t]" << endl << "}" << endl;
 
-	setlocale(LC_NUMERIC, old_num_locale);
-	g_free(old_num_locale);
 	g_output_stream_write (reinterpret_cast <GOutputStream *> (output), file.str ().c_str (), file.str ().size (), NULL, &error);
 	if (error) {
 		cerr << "gio error: " << error->message << endl;
