@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/document.h 
  *
- * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -42,12 +42,19 @@ class Dialog;
 class Molecule;
 class Residue;
 
+typedef enum {
+	ActionException,
+	ActionDelete,
+	ActionIgnore
+} Action;		
+
 class PendingTarget
 {
 public:
 	Object *parent;
 	Object *owner;
 	Object **target;
+	Action action;
 };
 
 /*!\class Document gcu/document.h
@@ -55,6 +62,7 @@ This class is the base document class.
 */
 class Document: public Object, virtual public DialogOwner
 {
+friend class Application;
 friend class Object;
 friend class Dialog;
 public:
@@ -160,7 +168,7 @@ using gcu::Document::Loaded(), and \a target is set to NULL.
 if \a owner is not NULL, post processing will call its OnLoaded() method.
 @return true if the target object was found.
 */
-	bool SetTarget (char const *id, Object **target, Object *parent, Object *owner = NULL) throw (std::runtime_error);
+	bool SetTarget (char const *id, Object **target, Object *parent, Object *owner = NULL, Action action = ActionException) throw (std::runtime_error);
 
 /*!
 Processes pending references resulting from failed calls to SetTarget().
@@ -172,6 +180,8 @@ Processes pending references resulting from failed calls to SetTarget().
 */
 	std::string Name ();
 
+	void ObjectLoaded (Object *obj);
+
 private:
 
 /*!
@@ -183,11 +193,12 @@ maintains a table to update links using Ids as identifiers. If Cache is set to t
 the table.
 GetNewId returns the translated id
 */
-	gchar* GetNewId (gchar* id, bool Cache = true);
+	char* GetNewId (char const *id, bool Cache = true);
 
 private:
 	std::map <std::string, std::string> m_TranslationTable;//used when Ids translations are necessary (on pasting...)
 	std::map <std::string, std::list <PendingTarget> > m_PendingTable;//used to set pointers to objects when loading does not occur in the ideal order
+	std::set<Object*> m_NewObjects;
 
 protected:
 /*!
@@ -207,7 +218,7 @@ The Application instance owning the document.
 @return a pointer to the Appication instance owning the ocument or NULL for
 an orphan document.
 */
-GCU_PROT_PROP (Application *, App)
+GCU_PROT_POINTER_PROP (Application, App)
 /*!\var m_Empty
 Tells if the document is empty or not.
 */
