@@ -2,9 +2,9 @@
 
 /* 
  * Gnome Chemistry Utils
- * molecule.h 
+ * molecule.cc 
  *
- * Copyright (C) 2001-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +24,7 @@
 
 #include "config.h"
 #include "molecule.h"
+#include "application.h"
 #include "atom.h"
 #include "bond.h"
 #include "chain.h"
@@ -31,6 +32,8 @@
 #include "document.h"
 #include "formula.h"
 #include "residue.h"
+#include <gsf/gsf-output-memory.h>
+#include <glib/gi18n-lib.h>
 #include <stack>
 
 using namespace std;
@@ -38,12 +41,16 @@ using namespace std;
 namespace gcu
 {
 
-Molecule::Molecule (TypeId Type): Object (Type)
+Molecule::Molecule (TypeId Type, ContentType ct): Object (Type)
 {
+	SetId ("m1");
+	m_Content = ct;
 }
 
-Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
+Molecule::Molecule (Atom* pAtom, ContentType ct): Object (MoleculeType)
 {
+	SetId ("m1");
+	m_Content = ct;
 	SetParent (pAtom->GetDocument ());
 	AddAtom (pAtom);
 	Chain* pChain = new Chain (this, pAtom); //will find the cycles
@@ -192,7 +199,8 @@ bool Molecule::operator== (Molecule const& molecule) const
 
 Molecule *Molecule::MoleculeFromFormula (Document *Doc, Formula const &formula, bool add_pseudo)
 {
-	Molecule *mol = reinterpret_cast <Molecule*> (Object::CreateObject ("molecule", Doc));
+	Application *app = Doc->GetApp ();
+	Molecule *mol = reinterpret_cast <Molecule*> (app->CreateObject ("molecule", Doc));
 	if (!mol)
 		return NULL;
 	stack <Atom*> atoms;
@@ -313,21 +321,21 @@ Molecule *Molecule::MoleculeFromFormula (Document *Doc, Formula const &formula,
 			delete mol;
 			return NULL;
 		}
-		atom = reinterpret_cast <Atom*> (Object::CreateObject ("pseudo-atom", mol));
-		bond = reinterpret_cast <Bond*> (CreateObject ("bond", mol));
+		atom = reinterpret_cast <Atom*> (app->CreateObject ("pseudo-atom", mol));
+		bond = reinterpret_cast <Bond*> (app->CreateObject ("bond", mol));
 		bond->SetOrder (1);
 		bond->ReplaceAtom (NULL, atom);
 		if (PendingAtoms.size () > 0)
 			atom = PendingAtoms.top ();
 		else {
-			atom = reinterpret_cast <Atom*> (Object::CreateObject ("atom", mol));
+			atom = reinterpret_cast <Atom*> (app->CreateObject ("atom", mol));
 			atom->SetZ (1);
 		}
 		bond->ReplaceAtom (NULL, atom);
 		atom->AddBond (bond);
 		PendingAtoms.pop ();
 	} else if (PendingHs + PendingAtoms.size () == 2) {
-		bond = reinterpret_cast <Bond*> (CreateObject ("bond", mol));
+		bond = reinterpret_cast <Bond*> (app->CreateObject ("bond", mol));
 		bond->ReplaceAtom (NULL, PendingAtoms.top ());
 		PendingAtoms.pop ();
 		bond->ReplaceAtom (NULL, PendingAtoms.top ());
@@ -363,4 +371,104 @@ Bond const *Molecule::GetNextBond(std::list<Bond*>::const_iterator &i) const
 	return (i == m_Bonds.end ())? NULL: *i;
 }
 
+void Molecule::SetName (char const *name, char const *convention)
+{
+	if (!convention)
+		convention = "Unknown";
+	m_Names[convention] = name;
+}
+
+char const *Molecule::GetName (char const *convention)
+{
+	if (m_Names.empty ())
+		return NULL;
+	if (!convention)
+		return (*m_Names.begin()).second.c_str ();
+	map <string, string>::iterator it = m_Names.find (convention);
+	return (it != m_Names.end ())? (*it).second.c_str (): NULL;
+}
+
+std::string Molecule::Name ()
+{
+	return _("Molecule");
+}
+
+std::string const &Molecule::GetCML ()
+{
+	if (m_CML.length () == 0) {
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->Save (output, "chemical/x-cml", this, m_Content);
+		size_t l = gsf_output_size (output);
+		if (l > 0)
+			m_CML.assign (reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output))), l);
+		g_object_unref (output);
+	}
+	return m_CML;
+}
+
+std::string const &Molecule::GetInChI ()
+{
+	if (m_InChI.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "inchi");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_InChI.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_InChI;
+}
+
+std::string const &Molecule::GetInChIKey ()
+{
+	if (m_InChIKey.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "inchi", "-xK");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_InChIKey.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_InChIKey;
+}
+
+std::string const &Molecule::GetSMILES ()
+{
+	if (m_SMILES.length () == 0) {
+		if (m_CML.length () == 0)
+			GetCML ();
+		GsfOutput *output = gsf_output_memory_new ();
+		GetDocument ()->GetApp ()->ConvertFromCML (m_CML.c_str (), output, "can");
+		size_t l = gsf_output_size (output);
+		if (l > 0) {
+			char const *res = reinterpret_cast <char const *> (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (output)));
+			while (res[l - 1] < ' ')
+				l--;
+			m_SMILES.assign (res, l);
+		}
+		g_object_unref (output);
+	}
+	return m_SMILES;
+}
+
+void Molecule::ResetIndentifiers ()
+{
+	m_CML.clear ();
+	m_InChI.clear ();
+	m_InChIKey.clear ();
+	m_SMILES.clear ();
+}
+
 }	//namespace gcu
