@@ -106,6 +106,7 @@ bool gcpTextTool::OnClicked ()
 	if (!m_pObject) {
 		gcp::Text *text = new gcp::Text(m_x0 / pTheme->GetZoomFactor (), m_y0 / pTheme->GetZoomFactor ());
 		m_pView->GetDoc ()->AddObject (text);
+		m_pView->GetDoc ()->AbortOperation ();
 		m_pObject = text;
 		create = true;
 	}
@@ -302,6 +303,8 @@ bool gcpTextTool::Unselect ()
 		if (m_Group) {
 			pOp = m_pView->GetDoc ()->GetCurrentOperation ();
 			if (pOp && dynamic_cast <gcp::ModifyOperation *> (pOp) == NULL) {
+				// looks like gcp::Document::AddObject sometimes add a new and unneeded AddOperation
+				// adding that as a security, but it looks like a bug.
 				m_pView->GetDoc ()->AbortOperation ();
 				pOp = NULL;
 			}
