@@ -31,14 +31,15 @@
 #include "document.h"
 #include "view.h"
 #include "window.h"
-#include "celldlg.h"
-#include "atomsdlg.h"
-#include "linesdlg.h"
-#include "sizedlg.h"
-#include "cleavagesdlg.h"
+#include <gcr/celldlg.h>
+#include <gcr/atomsdlg.h>
+#include <gcr/linesdlg.h>
+#include <gcr/sizedlg.h>
+#include <gcr/cleavagesdlg.h>
 #include "globals.h"
 #include <gcu/objprops.h>
 #include <gcu/filechooser.h>
+#include <gcu/xml-utils.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include <clocale>
@@ -89,19 +90,19 @@ void gcDocument::Define (unsigned nPage)
 {
 	switch (nPage) {
 	case 0:
-		new gcCellDlg (dynamic_cast <gcApplication *> (m_App), this);
+		new gcr::CellDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 1:
-		new gcAtomsDlg (dynamic_cast <gcApplication *> (m_App), this);
+		new gcr::AtomsDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 2:
-		new gcLinesDlg (dynamic_cast <gcApplication *> (m_App), this);
+		new gcr::LinesDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 3:
-		new gcSizeDlg (dynamic_cast <gcApplication *> (m_App), this);
+		new gcr::SizeDlg (dynamic_cast <gcApplication *> (m_App), this);
 		break;
 	case 4:
-		new gcCleavagesDlg (dynamic_cast <gcApplication *> (m_App), this);
+		new gcr::CleavagesDlg (m_App, this);
 		break;
 	}
 }
@@ -121,48 +122,6 @@ void gcDocument::UpdateAllViews()
 	}
 }
 
-void gcDocument::GetSize(gdouble* xmin, gdouble* xmax, gdouble* ymin, gdouble* ymax, gdouble* zmin, gdouble* zmax)
-{
-	*xmin = m_xmin;
-	*xmax = m_xmax;
-	*ymin = m_ymin;
-	*ymax = m_ymax;
-	*zmin = m_zmin;
-	*zmax = m_zmax;
-}
-
-void gcDocument::SetSize(gdouble xmin, gdouble xmax, gdouble ymin, gdouble ymax, gdouble zmin, gdouble zmax)
-{
-	m_xmin = xmin;
-	m_xmax = xmax;
-	m_ymin = ymin;
-	m_ymax = ymax;
-	m_zmin = zmin;
-	m_zmax = zmax;
-}
-
-void gcDocument::GetCell (gcr::Lattice *lattice, gdouble *a, gdouble *b, gdouble *c, gdouble *alpha, gdouble *beta, gdouble *gamma)
-{
-	*lattice = m_lattice;
-	*a = m_a;
-	*b = m_b;
-	*c = m_c;
-	*alpha = m_alpha;
-	*beta = m_beta;
-	*gamma = m_gamma;
-}
-
-void gcDocument::SetCell (gcr::Lattice lattice, gdouble a, gdouble b, gdouble c, gdouble alpha, gdouble beta, gdouble gamma)
-{
-	m_lattice = lattice;
-	m_a = a;
-	m_b = b;
-	m_c = c;
-	m_alpha = alpha;
-	m_beta = beta;
-	m_gamma = gamma;
-}
-
 void gcDocument::SetFileName (const string &filename)
 {
 	GFile *file = g_file_new_for_uri (filename.c_str ());
@@ -240,11 +199,8 @@ void gcDocument::Save() const
 		if (!g_date_valid (&m_CreationDate))
 			g_date_set_time_t (&const_cast <gcDocument *> (this)->m_CreationDate, time (NULL));
 		g_date_set_time_t (&const_cast <gcDocument *> (this)->m_RevisionDate, time (NULL));
-		gchar tmp[64];
-		g_date_strftime (tmp, sizeof (tmp), "%m/%d/%Y", &m_CreationDate);
-		xmlNewProp (xml->children, (xmlChar*) "creation", (xmlChar*) tmp);
-		g_date_strftime (tmp, sizeof (tmp), "%m/%d/%Y", &m_RevisionDate);
-		xmlNewProp (xml->children, (xmlChar*) "revision", (xmlChar*) tmp);
+		gcu::WriteDate (xml->children, "creation", &m_CreationDate);
+		gcu::WriteDate (xml->children, "revision", &m_RevisionDate);
 		xmlNodePtr node;
 
 		if (m_Title.length () > 0) {
@@ -398,13 +354,11 @@ bool gcDocument::Load (const string &filename)
 
 void gcDocument::ParseXMLTree(xmlNode* xml)
 {
-	char *old_num_locale, *txt;
+	char *txt;
 	xmlNodePtr node;
 	bool bViewLoaded = false;
 
 	Reinit();
-	old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-	setlocale(LC_NUMERIC, "C");
 	//look for generator node
 	unsigned version = 0xffffff , major, minor, micro;
 	node = xml->children;
@@ -420,22 +374,9 @@ void gcDocument::ParseXMLTree(xmlNode* xml)
 		g_free (m_Comment);
 		m_Comment = NULL;
 	}
-	g_date_clear (&m_CreationDate, 1);
-	g_date_clear (&m_RevisionDate, 1);
-	txt = (char*) xmlGetProp (xml, (xmlChar*) "creation");
-	if (txt) {
-		g_date_set_parse (&m_CreationDate, txt);
-		if (!g_date_valid (&m_CreationDate))
-			g_date_clear (&m_CreationDate, 1);
-		xmlFree (txt);
-	}
-	txt = (char*) xmlGetProp (xml, (xmlChar*) "revision");
-	if (txt) {
-		g_date_set_parse (&m_RevisionDate, txt);
-		if (!g_date_valid(&m_RevisionDate))
-			g_date_clear(&m_RevisionDate, 1);
-		xmlFree (txt);
-	}
+
+	gcu::ReadDate (xml, "creation", &m_CreationDate);
+	gcu::ReadDate (xml, "revision", &m_RevisionDate);
 
 	node = GetNodeByName (xml, "title");
 	if (node) {
@@ -514,7 +455,7 @@ void gcDocument::ParseXMLTree(xmlNode* xml)
 				txt = (char*) xmlGetProp (node, (xmlChar*) "fixed");
 				if (txt) {
 					if (!strcmp (txt, "true"))
-						m_bFixedSize = true;
+						SetFixedSize (true);
 					xmlFree (txt);
 				}
 			} else if (!strcmp((gchar*)node->name, "atom")) {
@@ -548,8 +489,6 @@ void gcDocument::ParseXMLTree(xmlNode* xml)
 			node = node->next;
 		}
 	}
-	setlocale (LC_NUMERIC, old_num_locale);
-	g_free (old_num_locale);
 	Update ();
 }
 
@@ -564,7 +503,7 @@ typedef struct {int n; std::list<gcr::Line*> l;} sLine;
 
 void gcDocument::OnExportVRML (const string &FileName) const
 {
-	char *old_num_locale, tmp[128];
+	char tmp[128];
 	double x0, x1, x2, x3, x4, x5;
 	int n = 0;
 	try {
@@ -580,8 +519,6 @@ void gcDocument::OnExportVRML (const string &FileName) const
 		}
 		std::map<std::string, sAtom>AtomsMap;
 		std::map<std::string, sLine>LinesMap;
-		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-		setlocale(LC_NUMERIC, "C");
 
 		file << "#VRML V2.0 utf8" << endl;
 		
@@ -681,8 +618,6 @@ void gcDocument::OnExportVRML (const string &FileName) const
 		//end of the world
 		file << "\t]" << endl << "}" << endl;
 
-		setlocale(LC_NUMERIC, old_num_locale);
-		g_free(old_num_locale);
 		g_output_stream_write (output, file.str ().c_str (), file.str ().size (), NULL, &error);
 		if (error) {
 			cerr << "gio error: " << error->message << endl;
